﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using System.Data;
using Telerik.ReportViewer.WinForms;
using Telerik.Reporting;
using Telerik.Reporting.Drawing;
using Telerik.WinControls.UI;

namespace GS.Print.TelerikReport.Aggregate
{
    public enum enumFunction
    {
        None = 0,
        Avg = 1,
        Count = 2,
        First = 3,
        Last = 4,
        Max = 5,
        Min = 6,
        Sum = 7,
    }

    public partial class Summary_Item
    {
        string c_strID = string.Empty;
        string c_strHeaderText = string.Empty;
        string c_strFooterText = string.Empty;
        string c_strFieldName = string.Empty;
        string c_strFormatString = string.Empty;
        GridAggregateFunction c_AggregateFunction = GridAggregateFunction.None;

        public Summary_Item(string FieldName, string FormatString, GridAggregateFunction AggregateFunction)
            : this(FieldName, FieldName, string.Empty, string.Empty, FormatString, AggregateFunction)
        {
        }

        public Summary_Item(string ID, string FieldName, string HeaderText, string FooterText, string FormatString, GridAggregateFunction AggregateFunction)
        {
            this.HeaderText = HeaderText;
            this.FooterText = FooterText;
            this.FieldName = FieldName;
            this.FormatString = FormatString;
            this.AggregateFunction = AggregateFunction;
        }

        // 因為有些專案, 不明原因不能參考GridAggregateFunction, 所以建立下列二建構式       Jul.02,09
        public Summary_Item(string FieldName, string FormatString, enumFunction AggregateFunction)
            : this(FieldName, FieldName, string.Empty, string.Empty, FormatString, AggregateFunction)
        {
        }

        public Summary_Item(string ID, string FieldName, string HeaderText, string FooterText, string FormatString, enumFunction AggregateFunction)
        {
            this.HeaderText = HeaderText;
            this.FooterText = FooterText;
            this.FieldName = FieldName;
            this.FormatString = FormatString;

            switch (AggregateFunction)
            {
                case enumFunction.None:
                    this.AggregateFunction = GridAggregateFunction.None;
                    break;

                case enumFunction.Avg:
                    this.AggregateFunction = GridAggregateFunction.Avg;
                    break;

                case enumFunction.Count:
                    this.AggregateFunction = GridAggregateFunction.Count;
                    break;

                case enumFunction.First:
                    this.AggregateFunction = GridAggregateFunction.First;
                    break;

                case enumFunction.Last:
                    this.AggregateFunction = GridAggregateFunction.Last;
                    break;

                case enumFunction.Max:
                    this.AggregateFunction = GridAggregateFunction.Max;
                    break;

                case enumFunction.Min:
                    this.AggregateFunction = GridAggregateFunction.Min;
                    break;

                case enumFunction.Sum:
                    this.AggregateFunction = GridAggregateFunction.Sum;
                    break;
            }
        }


        #region Properties

        public string ID
        {
            get { return c_strID; }
            set { c_strID = value; }
        }

        public string HeaderText
        {
            get { return c_strHeaderText; }
            set { c_strHeaderText = value; }
        }

        public string FooterText
        {
            get { return c_strFooterText; }
            set { c_strFooterText = value; }
        }

        public string FieldName
        {
            get { return c_strFieldName; }
            set { c_strFieldName = value; }
        }

        public string FormatString
        {
            get { return c_strFormatString; }
            set { c_strFormatString = value; }
        }

        public GridAggregateFunction AggregateFunction
        {
            get { return c_AggregateFunction; }
            set { c_AggregateFunction = value; }
        }

        #endregion
    }

    // Jul.01.09
    public partial class Summary
    {
        decimal c_decIncrease = 1;
        decimal c_decResult = 0;
        string c_strID = string.Empty;

        string c_strHeaderText = string.Empty;
        string c_strFooterText = string.Empty;
        string c_strFieldName = string.Empty;
        string c_strFormatString = string.Empty;
        GridAggregateFunction c_AggregateFunction = GridAggregateFunction.None;


        #region New, Load, Initial

        public Summary()
        {
        }

        public Summary(string ID)
            : this(ID, string.Empty)
        {
        }

        public Summary(string ID, string FieldName)
            : this(ID, FieldName, string.Empty, string.Empty, string.Empty)
        {
        }

        public Summary(string ID, string FieldName, GridAggregateFunction AggregateFunction)
            : this(ID, FieldName, string.Empty, string.Empty, string.Empty, AggregateFunction)
        {
        }

        public Summary(string ID, string FieldName, string HeaderText, string FooterText, string FormatString)
            : this(ID, FieldName, HeaderText, FooterText, FormatString, GridAggregateFunction.Sum)
        {
        }

        public Summary(string ID, string FieldName, string HeaderText, string FooterText, string FormatString, GridAggregateFunction AggregateFunction)
        {
            this.ID = ID;
            this.FieldName = FieldName;
            this.HeaderText = HeaderText;
            this.FooterText = FooterText;
            this.FormatString = FormatString;
            this.AggregateFunction = AggregateFunction;
        }

        public Summary(Summary_Item SummaryItem)
            : this(SummaryItem.ID, SummaryItem.FieldName, SummaryItem.HeaderText, SummaryItem.FooterText, SummaryItem.FormatString, SummaryItem.AggregateFunction)
        {
        }

        public void Initial()
        {
            this.Initial(0);
        }

        public void Initial(Decimal InitialValue)
        {
            c_decResult = InitialValue;
        }

        #endregion


        #region Properties

        public string ID
        {
            get { return c_strID; }
            set { c_strID = value; }
        }

        public string HeaderText
        {
            get { return c_strHeaderText; }
            set { c_strHeaderText = value; }
        }

        public string FooterText
        {
            get { return c_strFooterText; }
            set { c_strFooterText = value; }
        }

        public string FieldName
        {
            get { return c_strFieldName; }
            set { c_strFieldName = value; }
        }

        public string FormatString
        {
            get { return c_strFormatString; }
            set { c_strFormatString = value; }
        }

        public GridAggregateFunction AggregateFunction
        {
            get { return c_AggregateFunction; }
            set { c_AggregateFunction = value; }
        }

        public decimal Increase
        {
            get { return c_decIncrease; }
            set { c_decIncrease = value; }
        }

        public decimal Result
        {
            get { return c_decResult; }
        }

        #endregion


        #region Public Method

        public Boolean Accumulate(object value)
        {
            Boolean blnResult = false;

            switch (this.AggregateFunction)
            {
                case GridAggregateFunction.Count:
                    blnResult = true;
                    // 如果是GridAggregateFunction.Count, 則加入this.Increase
                    c_decResult += this.Increase;
                    break;

                case GridAggregateFunction.Avg:
                    // Implement
                    break;

                case GridAggregateFunction.First:
                    // Implement
                    break;

                case GridAggregateFunction.Last:
                    // Implement
                    break;

                case GridAggregateFunction.Max:
                    // Implement
                    break;

                case GridAggregateFunction.Min:
                    // Implement
                    break;

                case GridAggregateFunction.Sum:
                    if (null != value)
                    {
                        Decimal decValue = 0;
                        if (Decimal.TryParse(value.ToString(), out decValue))
                        {
                            blnResult = true;
                            this.Accumulate(decValue);
                        }
                    }
                    break;

                case GridAggregateFunction.None:
                default:
                    break;
            }

            return blnResult;
        }

        public void Accumulate(Decimal value)
        {
            switch (this.AggregateFunction)
            {
                case GridAggregateFunction.Count:
                    c_decResult += this.Increase;
                    break;

                case GridAggregateFunction.Avg:
                    // Implement
                    break;

                case GridAggregateFunction.First:
                    // Implement
                    break;

                case GridAggregateFunction.Last:
                    // Implement
                    break;

                case GridAggregateFunction.Max:
                    // Implement
                    break;

                case GridAggregateFunction.Min:
                    // Implement
                    break;

                case GridAggregateFunction.Sum:
                    c_decResult += value;
                    break;

                case GridAggregateFunction.None:
                default:
                    break;
            }
        }

        #endregion
   
    }// Summary

}
